package cn.com.qmth.im.multithread;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;

import cn.com.qmth.im.bean.StatusException;

public abstract class Consumer<T> extends Thread {

    private static Logger logger = LogManager.getLogger(Consumer.class);

    private Basket basket;

    private List<Object> ret = new ArrayList<>();

    public Consumer() {
    }

    @Override
    public void run() {
        try {
            while (true) {
                // 先判断是否有异常结束
                if (basket.isExcuteError()) {
                    break;
                }
                // 取消费数据
                Object o = basket.consume();
                // 判断消费数据是否是结束
                if (o instanceof EndObject) {
                    break;
                }
                @SuppressWarnings("unchecked")
                T t = (T) o;
                // 消费数据实现
                consume(t);
                // if (basket.getTotal() > 0) {
                // basket.updateProcess(1);
                // processInfo();
                // }
            }
        } catch (StatusException e) {
            logger.error(e.getMessage(), e);
            addMsg(e.getMessage());
            basket.setExcuteError(true);
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
            basket.setExcuteError(true);
        } finally {
            basket.countDown();
        }
    }

    protected void updateProcess(int count) {
        if (basket.getTotal() > 0) {
            basket.updateProcess(count);
            processInfo();
        }
    }

    protected abstract void consume(T t);

    protected void setBasket(Basket basket) {
        this.basket = basket;
    }

    protected Basket getBasket() {
        return this.basket;
    }

    protected void addMsg(String msg) {
        this.basket.addMsg(msg);
    }

    protected void processInfo() {
        logger.info(basket.getTaskName() + " 处理进度" + basket.getProgress());
    }

    public List<Object> getRet() {
        return ret;
    }

    public void addRet(Object r) {
        ret.add(r);
    }
}
